package org.springblade.modules.business.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springblade.common.base.BaseEntity;

import javax.validation.constraints.NotNull;
import java.util.Date;

/**
 * 售货设备表实体类
 *
 * @author zzyd
 * @since 2024-07-16
 */
@Data
@TableName("t_facility")
@EqualsAndHashCode(callSuper = true)
@ApiModel(value = "Facility对象", description = "售货设备表")
public class Facility extends BaseEntity {

	private static final long serialVersionUID = 1L;

	/**
	* 设备编号
	*/
	@ApiModelProperty(value = "设备编号")
	private String facilityNo;
	/**
	* 商家id
	*/
	@ApiModelProperty(value = "商家id")
	@NotNull(message = "请选择设备商家")
	private Long merchantId;
	/**
	 * 门店id
	 */
	@ApiModelProperty(value = "门店id")
	private Long shopId;
	/**
	* 设备名称
	*/
	@ApiModelProperty(value = "设备名称")
	private String facilityName;
	/**
	* 设备型号id
	*/
	@ApiModelProperty(value = "设备型号id")
	private Long facilityType;
	/**
	 * 设备地址
	 */
	@ApiModelProperty(value = "设备地址")
	private String address;
	/**
	* 设备经度
	*/
	@ApiModelProperty(value = "设备经度")
	private String longitude;
	/**
	* 设备纬度
	*/
	@ApiModelProperty(value = "设备纬度")
	private String latitude;
	/**
	 * 使用状态
	 */
	@ApiModelProperty(value = "使用状态StatusEnum，1启用，0禁用")
	private Integer useStatus;
	/**
	* 设备状态
	*/
	@ApiModelProperty(value = "设备状态FacilityStatusEnum，1在线，2离线，3故障")
	private Integer facilityStatus;
	/**
	* 最新上传数据时间
	*/
	@ApiModelProperty(value = "最新上传数据时间")
	private Date heartbeatTime;
	/**
	* 添加时间
	*/
	@ApiModelProperty(value = "添加时间")
	private Date addTime;

	/**
	 * 设备图片
	 */
	@TableField(exist = false)
	@ApiModelProperty(value = "设备图片")
	private String facilityImage;

	/**
	 * 设备二维码
	 */
	@TableField(exist = false)
	@ApiModelProperty(value = "设备二维码")
	private String qrCode;

}
